//
//  GameBoxLayer.m
//  PeerPong
//
//  Created by Volker Tietz on 12/3/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "GameBoxLayer.h"
#import "PeerPongGameplay.h"
#import "PeerPongPlayer.h"

@implementation GameBoxLayer

@synthesize leftEdge, rightEdge, topEdge, bottomEdge;

- (id)init {
    self = [super init];
    if (self) {
        gameplay = [PeerPongGameplay sharedInstance];
        
        // Define corners
        float maxY = gameplay.windowSize.height-gameplay.GROUND_BOX_PADDING;
        float maxX = gameplay.windowSize.width-gameplay.GROUND_BOX_PADDING;
        
        bottomLeftCorner = b2Vec2(gameplay.GROUND_BOX_PADDING/gameplay.PTM_RATIO,
                                  gameplay.GROUND_BOX_PADDING/gameplay.PTM_RATIO);
        
        bottomRightCorner = b2Vec2(maxX/gameplay.PTM_RATIO, 
                                   gameplay.GROUND_BOX_PADDING/gameplay.PTM_RATIO);
        
        topLeftCorner = b2Vec2(gameplay.GROUND_BOX_PADDING/gameplay.PTM_RATIO, 
                               maxY/gameplay.PTM_RATIO);
        
        topRightCorner = b2Vec2(maxX/gameplay.PTM_RATIO, 
                                maxY/gameplay.PTM_RATIO);
    
        [self createGameBox];
    }
    return self;
}


-(void)createGameBox {
    
    // Create edges around the entire screen
    b2BodyDef groundBodyDef;
    groundBodyDef.position.Set(0,0);
    gameplay.groundBody = gameplay.world->CreateBody(&groundBodyDef);
    
    groundBoxDef.shape = &groundBox;
    
    [self addEdgeAndBarForSlotNumber:0];
    [self addEdgeAndBarForSlotNumber:1];
    [self addEdgeAndBarForSlotNumber:2];
    [self addEdgeAndBarForSlotNumber:3];
}

-(void)addEdgeAndBarForSlotNumber:(int)aSlotNumber {
    switch (aSlotNumber) {
        case 0:
            // Bottom edge
            groundBox.SetAsEdge(bottomLeftCorner, bottomRightCorner);
            self.bottomEdge = gameplay.groundBody->CreateFixture(&groundBoxDef);
            break;
        case 1:
            // Top bar
            topBar = [CCSprite spriteWithFile:@"bar_top_blue_924.png"];
            topBar.position = CGPointMake(gameplay.windowSize.width/2, gameplay.windowSize.height-25);
            [self addChild: topBar];
            [topBar runAction:[CCFadeIn actionWithDuration:0.5]];
            
            // Top edge
            groundBox.SetAsEdge(topLeftCorner, topRightCorner);
            self.topEdge = gameplay.groundBody->CreateFixture(&groundBoxDef);
            break;
        case 2:          
            // Left bar
            leftBar = [CCSprite spriteWithFile:@"bar_left_yellow_50.png"];
            leftBar.position = CGPointMake(25, (gameplay.windowSize.height/2)+25);
            [self addChild: leftBar];
            [leftBar runAction:[CCFadeIn actionWithDuration:0.5]];
            
            // Left edge
            groundBox.SetAsEdge(bottomLeftCorner, topLeftCorner);
            self.leftEdge = gameplay.groundBody->CreateFixture(&groundBoxDef);
            break;
        case 3:
            // Right bar
            rightBar = [CCSprite spriteWithFile:@"bar_right_red_50.png"];
            rightBar.position = CGPointMake(gameplay.windowSize.width-25, (gameplay.windowSize.height/2)+25);
            [self addChild: rightBar];
            [rightBar runAction:[CCFadeIn actionWithDuration:0.5]];

            // Right edge
            groundBox.SetAsEdge(topRightCorner, bottomRightCorner);
            self.rightEdge = gameplay.groundBody->CreateFixture(&groundBoxDef);
            break;
            
        default:
            break;
    }
}

-(void)removeEdgeAndBarForPlayer:(PeerPongPlayer *)aPlayer {
    switch ([aPlayer.slot intValue]) {
        case 0:            
            gameplay.groundBody->DestroyFixture(self.bottomEdge);
            break;
        case 1:
            [self removeChild:topBar cleanup:YES];
            gameplay.groundBody->DestroyFixture(self.topEdge);
            break;
        case 2:
            [self removeChild:leftBar cleanup:YES];
            gameplay.groundBody->DestroyFixture(self.leftEdge);
            break;
        case 3:
            [self removeChild:rightBar cleanup:YES];
            gameplay.groundBody->DestroyFixture(self.rightEdge);
            break;
            
        default:
            break;
    }
}

- (void)dealloc {
    [super dealloc];
}

@end
