//
//  PeerPongGameplay.h
//  PeerPong
//
//  Created by Volker Tietz on 11/23/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AVFoundation/AVFoundation.h>
#import "cocos2d.h"
#import "Box2D.h"
@class PeerPongServer;
@class MainScene;
@class WelcomeLayer;
@class Command;
@class GameOver;
@class Hello;
@class GCDAsyncSocket;
@class UnicornServer;
@class PeerPongPlayer;

#define kSoundPlayerJoined 1
#define kSoundBallOut 2
#define kSoundPlayerLeft 3
#define kSoundBallOnPaddle 4
#define kSoundBallOnWall 5
#define kSoundPaddleOnPaddle 6
#define kSoundPaddleOnWall 7


@interface PeerPongGameplay : NSObject {
    PeerPongServer *peerPongServer;
    NSString *serverName;
        
    CCScene *currentScene;
    
    NSMutableDictionary *players;
    NSMutableArray *playerQueue;
    
    b2World *world;
    b2Body *groundBody;
    
    CGSize windowSize;
    NSString *fontName;
    int defaultFontSize;
    
    float PTM_RATIO;
    float GROUND_BOX_PADDING;
        
    BOOL gameIsRunning;
    
    int ballTag;
    int queueSpriteTag;
    
    NSLock *playerLock;
    
    PeerPongPlayer *recentlyAskedPlayer;
}

@property (nonatomic, retain) PeerPongServer *peerPongServer;
@property (nonatomic, retain) NSString *serverName;

@property (nonatomic,retain) NSMutableDictionary *players;
@property (nonatomic,retain) NSMutableArray *playerQueue;

@property (nonatomic) b2World *world;
@property (nonatomic) b2Body *groundBody;

@property (nonatomic) CGSize windowSize;
@property (nonatomic,retain) NSString *fontName;
@property (nonatomic) int defaultFontSize;

@property (nonatomic) float PTM_RATIO;
@property (nonatomic) float GROUND_BOX_PADDING;

@property (nonatomic) BOOL gameIsRunning;

@property (nonatomic) int ballTag;
@property (nonatomic) int queueSpriteTag;
           
+(PeerPongGameplay *)sharedInstance;

-(void)initiateGame;
-(void)destroyGame;
-(void)startGame;

-(void)startServer;
-(void)stopServer;

-(void)playSound:(int)aSound;

-(void)receivedCommand:(Command *)command fromClient:(GCDAsyncSocket *)client;
-(void)receivedHello:(Hello *)hello fromClient:(GCDAsyncSocket *)client;
-(void)receivedIAmReadyForSlot:(NSNumber*)aSlot fromClient:(GCDAsyncSocket *)client;
-(void)receivedDisconnectFromClient:(GCDAsyncSocket *)client;

-(PeerPongPlayer *)addPlayerWithHello:(Hello *)aHello andSocket:(GCDAsyncSocket*)socket;
-(PeerPongPlayer *)movePlayerFromQueueToSlot:(NSNumber *)aSlot WithSocket:(GCDAsyncSocket *)aSocket;
-(void)movePlayerFromSlotToQueue:(PeerPongPlayer *)aPlayer;
-(NSNumber *)getFreeSlotForPlayer;
-(void)removePlayerBySocket:(GCDAsyncSocket *)aSocket;
-(PeerPongPlayer *)getPlayerSlotBySocket:(GCDAsyncSocket *)aSocket;
-(void)askFirstPlayerForReadiness;
-(void)sendPlayerGoodBye:(PeerPongPlayer *)aPlayer;
-(void)sendPlayerGameOver:(PeerPongPlayer *)aPlayer;

-(void)popMessage:(NSString*)aMessage;

-(void)ballIsOnPosition:(CGPoint)aPosition;
-(void)ballWentAway:(CGPoint)aPosition;

@end
