//
//  PeerPongPaddle.m
//  PeerPong
//
//  Created by Volker Tietz on 12/3/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "PeerPongPaddle.h"
#import "PeerPongGameplay.h"
#import "PeerPongPlayer.h"

@implementation PeerPongPaddle

@synthesize sprite, playerName, playerNameShadow, playerLivesLeft;

-(id)initWithPlayer:(PeerPongPlayer *)aPlayer {
    self = [super init];
    if (self) {
        gameplay = [PeerPongGameplay sharedInstance];
        player = aPlayer;
        
        paddlePositions = [NSMutableArray arrayWithCapacity:4];
        [paddlePositions addObject:[NSNumber numberWithInt:100]]; 
        [paddlePositions addObject:[NSNumber numberWithInt:gameplay.windowSize.height - 50]]; 
        [paddlePositions addObject:[NSNumber numberWithInt:50]];
        [paddlePositions addObject:[NSNumber numberWithInt:gameplay.windowSize.width - 50]];
        
        spriteFiles = [NSMutableArray arrayWithCapacity:4];
        [spriteFiles addObject:@"paddle_bottom_green_153.png"];
        [spriteFiles addObject:@"paddle_top_blue_153.png"];
        [spriteFiles addObject:@"paddle_left_yellow_60.png"];
        [spriteFiles addObject:@"paddle_right_red_60.png"];
        
        paddlePosition = [[paddlePositions objectAtIndex:[player.slot intValue]] floatValue];
        self.sprite = [CCSprite spriteWithFile:[spriteFiles objectAtIndex:[player.slot intValue]]];
        
        int livesY = 35;
        int nameY = 10;
        int livesX = sprite.contentSize.width/2;
        int nameX = sprite.contentSize.width/2;
        int rotation = 0;
        
        switch ([player.slot intValue]) {
            case 1:
                livesY = 25;
                nameY = 50;
                break;
            case 2:
                livesY = sprite.contentSize.height/2;
                nameY = sprite.contentSize.height/2;
                livesX = 35;
                nameX = 10;
                rotation = 90;
                break;
            case 3:
                livesY = sprite.contentSize.height/2;
                nameY = sprite.contentSize.height/2;
                livesX = 25;
                nameX = 50;

                rotation = 270;
                break;
            default:
                break;
        }
        
        self.playerLivesLeft = [CCLabelTTF labelWithString:[NSString stringWithFormat:@"%d", player.livesLeft] 
                                                  fontName:gameplay.fontName fontSize:16];
        self.playerLivesLeft.position = ccp(livesX, livesY);
        
        self.playerNameShadow = [CCLabelTTF labelWithString:player.name fontName:gameplay.fontName fontSize:16];
        self.playerNameShadow.color = ccc3(0, 0, 0);
        
        self.playerName = [CCLabelTTF labelWithString:player.name fontName:gameplay.fontName fontSize:16];
        self.playerName.position = ccp(nameX, nameY);
        self.playerName.rotation = rotation;

        self.playerNameShadow = [CCLabelTTF labelWithString:player.name fontName:gameplay.fontName fontSize:16];
        self.playerNameShadow.position = ccp(self.playerName.position.x+1, self.playerName.position.y-1);
        self.playerNameShadow.color = ccc3(0, 0, 0);
        self.playerNameShadow.rotation = rotation;
        
        boxWidthForPaddle = gameplay.windowSize.width-(2*gameplay.GROUND_BOX_PADDING);
        boxHeightForPaddle = gameplay.windowSize.height-(2*gameplay.GROUND_BOX_PADDING);
    }
    return self;

}

-(void)updateLivesLeftLabel {
    self.playerLivesLeft.string = [NSString stringWithFormat:@"%d", player.livesLeft];
}
    
-(void)addPhysics {
    // Create paddle body
    b2BodyDef bodyDef;
    bodyDef.type = b2_dynamicBody;
    
    b2Vec2 pos = [self getLocation];
    bodyDef.position.Set(pos.x, pos.y);
    
    bodyDef.userData = sprite;
    body = gameplay.world->CreateBody(&bodyDef);
    
    // Create paddle shape
    b2PolygonShape shape;
    shape.SetAsBox((sprite.contentSize.width-20)/gameplay.PTM_RATIO/2, 
                           (sprite.contentSize.height-20)/gameplay.PTM_RATIO/2);
    
    // Create shape definition and add to body
    b2FixtureDef shapeDef;
    shapeDef.shape = &shape;
    shapeDef.density = 10.0f;
    shapeDef.friction = 0.4f;
    shapeDef.restitution = 0.1f;
    fixture = body->CreateFixture(&shapeDef);
    
    // Restrict paddle along the x axis
    b2PrismaticJointDef jointDef;
    jointDef.collideConnected = true;
    jointDef.Initialize(body, gameplay.groundBody, 
                        body->GetWorldCenter(), [self getAxis]);
    joint = gameplay.world->CreateJoint(&jointDef);
}

-(void)removePhysics {
    // If user didn't move his paddle mouseJoint is NULL
    if (mouseJoint != NULL) {
        gameplay.world->DestroyJoint(mouseJoint);
    }
    gameplay.world->DestroyJoint(joint);
    body->DestroyFixture(fixture);
    gameplay.world->DestroyBody(body);
}

-(float)getTranslatedOffsetForX {
    return (boxWidthForPaddle * player.offset) + gameplay.GROUND_BOX_PADDING;
}
       
-(float)getTranslatedOffsetForY {
   return (boxHeightForPaddle * player.offset) + gameplay.GROUND_BOX_PADDING;
}
       
-(b2Vec2)getLocation {
    b2Vec2 location;
    if([self isHorizontal]) {
        location = b2Vec2([self getTranslatedOffsetForX]/gameplay.PTM_RATIO, paddlePosition/gameplay.PTM_RATIO);
    } else {
        location = b2Vec2(paddlePosition/gameplay.PTM_RATIO, [self getTranslatedOffsetForY]/gameplay.PTM_RATIO);
    }
    return location;
}
       
-(b2Vec2)getAxis {
    if([self isHorizontal]) {
        b2Vec2 axis(1.0f, 0.0f);
        return axis;
    } else {
        b2Vec2 axis(0.0f, 1.0f);
        return axis;
    }
}
       
-(void)update {
    if (mouseJoint == NULL) {
        b2Vec2 locationWorld = [self getLocation];
        
        if (fixture->TestPoint(locationWorld)) {
            b2MouseJointDef md;
            md.bodyA = gameplay.groundBody;
            md.bodyB = body;
            md.target = locationWorld;
            md.collideConnected = true;
            md.maxForce = 1000.0f * body->GetMass();
            
            mouseJoint = (b2MouseJoint *)gameplay.world->CreateJoint(&md);
            body->SetAwake(true);
        }
    } else {
        b2Vec2 locationWorld = [self getLocation];
        mouseJoint->SetTarget(locationWorld);
    }

}

-(BOOL)isHorizontal {
    if([player.slot intValue] == 0 || [player.slot intValue] == 1) {
        return YES;
    } else {
        return NO;
    }
}

@end
